import os

from django.core.wsgi import get_wsgi_application

# Set the default settings module for the 'django' program.
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "test_project.settings")

# Get the WSGI application for the Django project.
application = get_wsgi_application()



# -*- coding: utf-8 -*-
"""
    django-notifications-rest
    ~~~~~
   A django notifications package exposed by Rest.
"""

# PEP 386-compliant version number: N.N[.N]+[{a|b|c|rc}N[.N]+][.postN][.devN]
__version__ = "1.0.2"

default_app_config = "notifications_rest.apps.NotificationsRestConfig"



import os
from pathlib import Path

# Base directory
BASE_DIR = Path(__file__).resolve().parent.parent

# Secret key
SECRET_KEY = "your-secret-key"

# Debug mode
DEBUG = True

# Allowed hosts
ALLOWED_HOSTS = []

# Installed apps
INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "rest_framework",
    "notifications",
    "notifications_rest",
]

# Middleware
MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

# Root URL configuration
ROOT_URLCONF = "test_project.urls"

# Templates
TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [os.path.join(BASE_DIR, "templates")],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]

# WSGI application
WSGI_APPLICATION = "test_project.wsgi.application"

# Database configuration
DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": BASE_DIR / "db.sqlite3",
    }
}

# Password validation
AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator",
    },
]

# Internationalization
LANGUAGE_CODE = "en-us"
TIME_ZONE = "UTC"
USE_I18N = True
USE_L10N = True
USE_TZ = True

# Static files
STATIC_URL = "/static/"

# Default primary key field type
DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"

# REST Framework configuration
REST_FRAMEWORK = {
    "DEFAULT_PERMISSION_CLASSES": [
        "rest_framework.permissions.IsAuthenticated",
    ],
    "DEFAULT_AUTHENTICATION_CLASSES": [
        "rest_framework.authentication.SessionAuthentication",
        "rest_framework.authentication.BasicAuthentication",
    ],
}

NOTIFICATIONS_API_ALLOW_ADD = True



from django.contrib.auth import get_user_model
from django.contrib.contenttypes.models import ContentType
from notifications.models import Notification
from rest_framework import serializers
from rest_framework.serializers import ModelSerializer, Serializer

UserModel = get_user_model()


class UserSerializer(ModelSerializer):
    id = serializers.IntegerField()

    class Meta:
        model = UserModel
        fields = ["id"]


class GenericSerializer(Serializer):
    pk = serializers.SerializerMethodField()
    content_type_name = serializers.SerializerMethodField(read_only=True)
    content_type_model = serializers.SerializerMethodField(read_only=True)
    content_type_app = serializers.SerializerMethodField(read_only=True)
    content_type_id = serializers.SerializerMethodField()
    url = serializers.SerializerMethodField(read_only=True)
    string = serializers.SerializerMethodField(read_only=True)

    class Meta:
        fields = "__all__"

    def get_obj(self, data):
        content_type = ContentType.objects.get(id=data["content_type_id"])
        model_class = content_type.model_class()
        return model_class.objects.get(pk=data["pk"])

    def to_internal_value(self, data):
        return self.get_obj(data)

    def get_pk(self, obj):
        return obj.pk

    def get_obj_content_type(self, obj):
        if not hasattr(self, "obj_content_type"):
            self.obj_content_type = ContentType.objects.get_for_model(obj)
        return self.obj_content_type

    def get_content_type_app(self, obj):
        return self.get_obj_content_type(obj).app_label

    def get_content_type_model(self, obj):
        return self.get_obj_content_type(obj).model

    def get_content_type_name(self, obj):
        return self.get_obj_content_type(obj).name

    def get_content_type_id(self, obj):
        return self.get_obj_content_type(obj).id

    def get_string(self, obj):
        return obj.__str__()

    def get_url(self, obj):
        try:
            return obj.get_absolute_url()
        except AttributeError:
            return ""


class NotificationSerializer(ModelSerializer):
    recipient = UserSerializer()
    actor = GenericSerializer()
    verb = serializers.CharField()
    level = serializers.CharField()
    description = serializers.CharField()
    timestamp = serializers.DateTimeField(read_only=True)
    unread = serializers.BooleanField()
    public = serializers.BooleanField()
    deleted = serializers.BooleanField()
    emailed = serializers.BooleanField()
    target = GenericSerializer(required=False)
    action_object = GenericSerializer(required=False)
    string = serializers.CharField(source="__str__", read_only=True)

    class Meta:
        model = Notification
        fields = [
            "id",
            "recipient",
            "actor",
            "target",
            "verb",
            "action_object",
            "level",
            "description",
            "unread",
            "public",
            "deleted",
            "emailed",
            "timestamp",
            "string",
        ]

    def create(self, validated_data):
        recipient_data = validated_data.pop("recipient")
        recipient = UserModel.objects.get_or_create(id=recipient_data["id"])
        notification = Notification.objects.create(recipient=recipient[0], **validated_data)
        return notification



from django.conf import settings
from django.urls import re_path as url
from rest_framework import routers

from .views import (
    AddNotification,
    AllNotification,
    AllNotificationCount,
    Delete,
    MarkAllAsRead,
    MarkAsRead,
    MarkAsUnread,
    NotificationViewSet,
    UnreadNotificationCount,
    UnreadNotificationsList,
)

router = routers.DefaultRouter()
router.register("", NotificationViewSet)

app_name = "notifications_rest"
urlpatterns = [
    url(r"^all/", AllNotification.as_view({"get": "list"}), name="all"),
    url(r"^unread/", UnreadNotificationsList.as_view({"get": "list"}), name="unread"),
    url(r"^mark-all-as-read/$", MarkAllAsRead.as_view(), name="mark_all_as_read"),
    url(r"^mark-as-read/(?P<slug>\d+)/$", MarkAsRead.as_view(), name="mark_as_read"),
    url(r"^mark-as-unread/(?P<slug>\d+)/$", MarkAsUnread.as_view(), name="mark_as_unread"),
    url(r"^delete/(?P<slug>\d+)/$", Delete.as_view(), name="delete"),
    url(r"^unread_count/$", UnreadNotificationCount.as_view(), name="unread_notification_count"),
    url(r"^all_count/$", AllNotificationCount.as_view(), name="all_notification_count"),
]

if getattr(settings, "NOTIFICATIONS_API_ALLOW_ADD", False):
    urlpatterns += [
        url(r"^add/", AddNotification.as_view(), name="add"),
    ]



from notifications.models import Notification
from rest_framework import status, viewsets
from rest_framework.generics import CreateAPIView
from rest_framework.mixins import ListModelMixin
from rest_framework.response import Response
from rest_framework.views import APIView
from rest_framework.viewsets import GenericViewSet

from .serializers import NotificationSerializer


class UnreadNotificationsList(ListModelMixin, GenericViewSet):
    serializer_class = NotificationSerializer

    def get_queryset(self, *args, **kwargs):
        return Notification.objects.filter(recipient_id=self.request.user.id, unread=True)


class MarkAllAsRead(APIView):
    serializer_class = NotificationSerializer

    def get(self, request, format=None):
        queryset = Notification.objects.filter(recipient_id=request.user.id, unread=True)
        queryset.update(unread=False)
        return Response({"code": "OK"}, status=status.HTTP_200_OK)


class NotificationMixin:
    def get_obj(self, request, *args, **kwargs):
        notification_id = kwargs.get("slug")
        queryset = Notification.objects.filter(recipient=request.user)
        return queryset.get(id=notification_id)


class MarkAsRead(NotificationMixin, APIView):
    serializer_class = NotificationSerializer

    def get(self, request, *args, **kwargs):
        notification_obj = self.get_obj(request, *args, **kwargs)
        notification_obj.unread = False
        notification_obj.save()
        return Response({"code": "OK"}, status=status.HTTP_200_OK)


class MarkAsUnread(NotificationMixin, APIView):
    serializer_class = NotificationSerializer

    def get(self, request, *args, **kwargs):
        notification_obj = self.get_obj(request, *args, **kwargs)
        notification_obj.unread = True
        notification_obj.save()
        return Response({"code": "OK"}, status=status.HTTP_200_OK)


class Delete(NotificationMixin, APIView):
    serializer_class = NotificationSerializer

    def delete(self, request, *args, **kwargs):
        notification_obj = self.get_obj(request, *args, **kwargs)
        notification_obj.delete()
        return Response({"code": "OK"}, status=status.HTTP_200_OK)


class AddNotification(CreateAPIView):
    serializer_class = NotificationSerializer

    def create(self, request, *args, **kwargs):
        response = super(AddNotification, self).create(request, *args, **kwargs)
        return response


class AllNotification(ListModelMixin, GenericViewSet):
    serializer_class = NotificationSerializer

    def get_queryset(self, *args, **kwargs):
        return Notification.objects.filter(recipient_id=self.request.user.id)


class UnreadNotificationCount(APIView):
    serializer_class = NotificationSerializer

    def get(self, request, *args, **kwargs):
        queryset = Notification.objects.filter(recipient_id=request.user.id, unread=True)
        count = queryset.count()
        data = {"unread_count": count}
        return Response(data)


class AllNotificationCount(APIView):
    serializer_class = NotificationSerializer

    def get(self, request, *args, **kwargs):
        queryset = Notification.objects.filter(recipient_id=request.user.id)
        count = queryset.count()
        data = {"all_count": count}
        return Response(data)


class NotificationViewSet(viewsets.ModelViewSet):
    serializer_class = NotificationSerializer
    queryset = Notification.objects.all()



from django.contrib.auth import get_user_model
from django.contrib.contenttypes.models import ContentType
from django.urls import reverse
from notifications.models import Notification
from rest_framework import status
from rest_framework.test import APIClient, APITestCase

User = get_user_model()


class NotificationTests(APITestCase):

    def setUp(self):
        self.user = User.objects.create_user(username="testuser", password="testpass")
        self.client = APIClient()
        self.client.login(username="testuser", password="testpass")
        self.notification = Notification.objects.create(
            recipient=self.user,
            actor=self.user,
            verb="test",
            description="test notification",
            unread=True,
            public=False,
            deleted=False,
            emailed=False,
        )

    def test_get_unread_notifications(self):
        url = reverse("notifications_rest:unread")
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.data), 1)

    def test_mark_all_as_read(self):
        url = reverse("notifications_rest:mark_all_as_read")
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.notification.refresh_from_db()
        self.assertFalse(self.notification.unread)

    def test_mark_as_read(self):
        url = reverse("notifications_rest:mark_as_read", kwargs={"slug": self.notification.id})
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.notification.refresh_from_db()
        self.assertFalse(self.notification.unread)

    def test_mark_as_unread(self):
        self.notification.unread = False
        self.notification.save()
        url = reverse("notifications_rest:mark_as_unread", kwargs={"slug": self.notification.id})
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.notification.refresh_from_db()
        self.assertTrue(self.notification.unread)

    def test_delete_notification(self):
        url = reverse("notifications_rest:delete", kwargs={"slug": self.notification.id})
        response = self.client.delete(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertFalse(Notification.objects.filter(id=self.notification.id).exists())

    def test_add_notification_minimal(self):
        url = reverse("notifications_rest:add")
        data = {
            "recipient": {"id": self.user.id},
            "actor": {
                "pk": self.user.id,
                "content_type_id": ContentType.objects.get_for_model(self.user).id,
            },
            "verb": "new",
            "description": "new notification",
            "unread": True,
            "public": False,
            "deleted": False,
            "level": "1",
            "emailed": False,
        }
        response = self.client.post(url, data, format="json")
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertEqual(response.data["recipient"]["id"], self.user.id)
        self.assertEqual(Notification.objects.count(), 2)

    def test_add_notification_full(self):
        url = reverse("notifications_rest:add")
        data = {
            "recipient": {"id": self.user.id},
            "actor": {
                "pk": self.user.id,
                "content_type_id": ContentType.objects.get_for_model(self.user).id,
            },
            "verb": "new",
            "description": "new notification",
            "unread": True,
            "public": False,
            "deleted": False,
            "level": "1",
            "emailed": False,
            "target": {
                "pk": self.user.id,
                "content_type_id": ContentType.objects.get_for_model(self.user).id,
            },
            "action_object": {
                "pk": self.user.id,
                "content_type_id": ContentType.objects.get_for_model(self.user).id,
            },
        }
        response = self.client.post(url, data, format="json")
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertEqual(response.data["recipient"]["id"], self.user.id)
        self.assertEqual(Notification.objects.count(), 2)

    def test_get_all_notifications(self):
        url = reverse("notifications_rest:all")
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.data), 1)

    def test_get_unread_notification_count(self):
        url = reverse("notifications_rest:unread_notification_count")
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data["unread_count"], 1)

    def test_get_all_notification_count(self):
        url = reverse("notifications_rest:all_notification_count")
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data["all_count"], 1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
""" Django notifications rest setup file for pip package """
import ast
import re

import setuptools

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup  # pylint: disable=no-name-in-module,import-error

_version_re = re.compile(r"__version__\s+=\s+(.*)")  # pylint: disable=invalid-name

with open("notifications_rest/__init__.py", "rb") as f:
    version = str(
        ast.literal_eval(_version_re.search(f.read().decode("utf-8")).group(1))  # pylint: disable=invalid-name
    )

setup(
    name="django-notifications-rest",
    version=version,
    description="A django notifications package exposed through API Rest",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    author="Yusniel Hidalgo Delgado",
    author_email="yhidalgo86@gmail.com",
    url="https://github.com/yhdelgado/django-notifications-rest",
    include_package_data=True,
    exclude_package_data={"": ["djangotools", "django_notifications_rest.egg-info"]},
    packages=setuptools.find_packages(exclude=["djangotools"]),
    python_requires=">=3.5",
    install_requires=[],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Framework :: Django",
        "Framework :: Django :: 2.2",
        "Framework :: Django :: 3.0",
        "Framework :: Django :: 3.1",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Utilities",
    ],
    keywords="django notifications github rest api",
    license="MIT",
)



from django.contrib import admin
from django.urls import include, path

urlpatterns = [
    path("notifications/", include("notifications_rest.urls"), name="notifications_rest"),
    path("admin/", admin.site.urls),
]
